==> 0.3.0.tar.gz: lib/src/mutation_tracking_list.dart <==
library disposable.mutation_tracking_list;

import 'dart:math';

import 'package:collection/wrappers.dart';

abstract class MutationTrackingList<SELF extends MutationTrackingList<SELF, E>,
    E> extends DelegatingList<E> {
  MutationTrackingList(List<E> base) : super(base);

  /// Invoked when a sequence of elements in [added] are added
  /// at [index].
  void elementsAdded(int index, Iterable<E> added);

  /// Invoked when a sequence of characters in [removed] are removed
  /// from [index].
  void elementsRemoved(int index, List<E> removed);

  /// Invoked when a sequence of characters in [removed] that
  /// were at [index] are replaced with those in [added].
  void elementsReplaced(int index, List<E> removed, Iterable<E> added);

  @override
  void set length(int newLength) {
    var oldLength = length;
    if (newLength > length) {
      super.length = newLength;
      elementsAdded(oldLength, sublist(oldLength, newLength));
    } else if (newLength < length) {
      var removed = sublist(newLength);
      super.length = newLength;
      elementsRemoved(newLength, removed);
    }
  }

  @override
  void operator []=(int index, E value) {
    var replaced = super[index];
    super[index] = value;
    elementsReplaced(index, [replaced], [value]);
  }

  @override
  void add(E value) {
    super.add(value);
    elementsAdded(length - 1, [value]);
  }

  @override
  void addAll(Iterable<E> added) {
    var index = length;
    super.addAll(added);
    elementsAdded(index, added);
  }

  @override
  void insert(int index, E element) {
    super.insert(index, element);
    elementsAdded(index, [element]);
  }

  @override
  void insertAll(int index, Iterable<E> iterable) {
    super.insertAll(index, iterable);
    elementsAdded(index, iterable);
  }

  @override
  void clear() {
    var removed = sublist(0);
    super.clear();
    elementsRemoved(0, removed);
  }

  @override
  bool remove(Object value) {
    var index = indexOf(value);
    if (index == -1) {
      return false;
    }
    removeAt(index);
    return true;
  }

  @override
  E removeAt(int index) {
    var removed = super.removeAt(index);
    elementsRemoved(index, [removed]);

    return removed;
  }

  @override
  E removeLast() {
    var removed = super.removeLast();
    elementsRemoved(length, [removed]);

    return removed;
  }

  @override
  void removeRange(int start, int end) {
    var removed = sublist(start, end);
    super.removeRange(start, end);
    elementsRemoved(start, removed);
  }

  @override
  void replaceRange(int start, int end, Iterable<E> replacement) {
    var removed = sublist(start, end);
    super.replaceRange(start, end, replacement);
    elementsReplaced(start, removed, replacement);
  }

  @override
  void setAll(int index, Iterable<E> iterable) {
    var removed = sublist(index, index + iterable.length);
    super.setAll(index, iterable);
    elementsReplaced(index, removed, iterable);
  }

  @override
  void setRange(int start, int end, Iterable<E> iterable, [int skipCount = 0]) {
    var removed = sublist(start, end);
    super.setRange(start, end, iterable, skipCount);
    if (skipCount > 0) {
      iterable = iterable.skip(skipCount);
    }
    elementsReplaced(start, removed, iterable);
  }

  @override
  void fillRange(int start, int end, [E fillValue]) {
    var removed = sublist(start, end);
    super.fillRange(start, end, fillValue);
    elementsReplaced(start, removed, getRange(start, end));
  }

  @override
  void removeWhere(bool test(E c)) {
    _filter(test, false);
  }

  @override
  void retainWhere(bool test(E c)) {
    _filter(test, true);
  }

  void _filter(bool test(E element), bool retainMatching) {
    var nextRetained = 0;
    var length = this.length;
    var firstRemovedInSuccession = -1;

    for (var i = 0; i < length; i++) {
      var element = this[i];
      if (test(element) == retainMatching) {
        if (firstRemovedInSuccession >= 0) {
          elementsRemoved(
              firstRemovedInSuccession, sublist(firstRemovedInSuccession, i));
          firstRemovedInSuccession = -1;
        }
        super[nextRetained++] = element;
      } else {
        // Removed
        if (firstRemovedInSuccession == -1) {
          firstRemovedInSuccession = i;
        }
      }
      if (length != this.length) {
        throw new ConcurrentModificationError(this);
      }
    }
    if (firstRemovedInSuccession != -1) {
      elementsRemoved(
          firstRemovedInSuccession, sublist(firstRemovedInSuccession, length));
    }
  }

  @override
  bool get isEmpty => length == 0;

  @override
  bool get isNotEmpty => length != 0;

  @override
  void shuffle([Random random]) {
    throw new UnsupportedError('shuffle not supported');
  }

  @override
  void sort([int compare(E a, E b)]) {
    throw new UnsupportedError('sort not supported');
  }
}

==> 0.3.1.tar.gz: lib/src/mutation_tracking_list.dart <==
library disposable.mutation_tracking_list;

import 'dart:math';

import 'package:quiver_collection/collection.dart';

abstract class MutationTrackingList<SELF extends MutationTrackingList<SELF, E>,
    E> extends DelegatingList<E> {
  final List<E> _base;

  List<E> get delegate => _base;

  MutationTrackingList(this._base);

  /// Invoked when a sequence of elements in [added] are added
  /// at [index].
  void elementsAdded(int index, Iterable<E> added);

  /// Invoked when a sequence of characters in [removed] are removed
  /// from [index].
  void elementsRemoved(int index, List<E> removed);

  /// Invoked when a sequence of characters in [removed] that
  /// were at [index] are replaced with those in [added].
  void elementsReplaced(int index, List<E> removed, Iterable<E> added);

  @override
  void set length(int newLength) {
    var oldLength = length;
    if (newLength > length) {
      super.length = newLength;
      elementsAdded(oldLength, sublist(oldLength, newLength));
    } else if (newLength < length) {
      var removed = sublist(newLength);
      super.length = newLength;
      elementsRemoved(newLength, removed);
    }
  }

  @override
  void operator []=(int index, E value) {
    var replaced = super[index];
    super[index] = value;
    elementsReplaced(index, [replaced], [value]);
  }

  @override
  void add(E value) {
    super.add(value);
    elementsAdded(length - 1, [value]);
  }

  @override
  void addAll(Iterable<E> added) {
    var index = length;
    super.addAll(added);
    elementsAdded(index, added);
  }

  @override
  void insert(int index, E element) {
    super.insert(index, element);
    elementsAdded(index, [element]);
  }

  @override
  void insertAll(int index, Iterable<E> iterable) {
    super.insertAll(index, iterable);
    elementsAdded(index, iterable);
  }

  @override
  void clear() {
    var removed = sublist(0);
    super.clear();
    elementsRemoved(0, removed);
  }

  @override
  bool remove(Object value) {
    var index = indexOf(value);
    if (index == -1) {
      return false;
    }
    removeAt(index);
    return true;
  }

  @override
  E removeAt(int index) {
    var removed = super.removeAt(index);
    elementsRemoved(index, [removed]);

    return removed;
  }

  @override
  E removeLast() {
    var removed = super.removeLast();
    elementsRemoved(length, [removed]);

    return removed;
  }

  @override
  void removeRange(int start, int end) {
    var removed = sublist(start, end);
    super.removeRange(start, end);
    elementsRemoved(start, removed);
  }

  @override
  void replaceRange(int start, int end, Iterable<E> replacement) {
    var removed = sublist(start, end);
    super.replaceRange(start, end, replacement);
    elementsReplaced(start, removed, replacement);
  }

  @override
  void setAll(int index, Iterable<E> iterable) {
    var removed = sublist(index, index + iterable.length);
    super.setAll(index, iterable);
    elementsReplaced(index, removed, iterable);
  }

  @override
  void setRange(int start, int end, Iterable<E> iterable, [int skipCount = 0]) {
    var removed = sublist(start, end);
    super.setRange(start, end, iterable, skipCount);
    if (skipCount > 0) {
      iterable = iterable.skip(skipCount);
    }
    elementsReplaced(start, removed, iterable);
  }

  @override
  void fillRange(int start, int end, [E fillValue]) {
    var removed = sublist(start, end);
    super.fillRange(start, end, fillValue);
    elementsReplaced(start, removed, getRange(start, end));
  }

  @override
  void removeWhere(bool test(E c)) {
    _filter(test, false);
  }

  @override
  void retainWhere(bool test(E c)) {
    _filter(test, true);
  }

  void _filter(bool test(E element), bool retainMatching) {
    var nextRetained = 0;
    var length = this.length;
    var firstRemovedInSuccession = -1;

    for (var i = 0; i < length; i++) {
      var element = this[i];
      if (test(element) == retainMatching) {
        if (firstRemovedInSuccession >= 0) {
          elementsRemoved(
              firstRemovedInSuccession, sublist(firstRemovedInSuccession, i));
          firstRemovedInSuccession = -1;
        }
        super[nextRetained++] = element;
      } else {
        // Removed
        if (firstRemovedInSuccession == -1) {
          firstRemovedInSuccession = i;
        }
      }
      if (length != this.length) {
        throw new ConcurrentModificationError(this);
      }
    }
    if (firstRemovedInSuccession != -1) {
      elementsRemoved(
          firstRemovedInSuccession, sublist(firstRemovedInSuccession, length));
    }
  }

  @override
  bool get isEmpty => length == 0;

  @override
  bool get isNotEmpty => length != 0;

  @override
  void shuffle([Random random]) {
    throw new UnsupportedError('shuffle not supported');
  }

  @override
  void sort([int compare(E a, E b)]) {
    throw new UnsupportedError('sort not supported');
  }
}
